% Image cropping by minimizing the rectangluar area of interest
% P: energy map (objectness, saliency, etc.), size <= 1000
% ratio: minimum percentage of total energy to be preserved
% aspect: aspect ratio of cropping rectangle (width/height), 0 for free size

function rect = MinAI(P, ratio, aspect)

    assert(min(P(:)) >= 0);
    assert(max(P(:)) <= 1);
    assert(max(size(P)) <= 1000);
    assert(max(size(ratio)) <= 100);
    assert(min(ratio(:)) > 0);
    assert(max(ratio(:)) < 1);
    assert(aspect >= 0);

    % rotate to achieve lower complexity
    isRot = 0;
    if size(P,2) > size(P,1)
        P = P';  isRot = 1;
        if(aspect~=0)
            aspect = 1/aspect;
        end
    end
    
    rect = zeros(length(ratio), 4);
    for i = 1:length(ratio)
        rect(i, :) = OPIC_internal(P, sum(P(:))*ratio(i), aspect);
    end
    
    if isRot>0
        rect = [rect(:, 2) rect(:, 1) rect(:, 4) rect(:, 3)];
    end
    
end

function min_rect = OPIC_internal(P, T, aspect)

    [H, W] = size(P);
    
    % get some integral maps
    xProjCum = [0 cumsum(sum(P,1))];
    int_row = [zeros(size(P,1), 1) cumsum(P,2)]; % integral map by row
    
    min_area = H*W + 1;
    min_sum = 0;
    min_rect = zeros(1,4);
    % loop all column pairs (i,j)
    for i = 1:size(P,2)

        idx = find(xProjCum>=xProjCum(i)+T, 1, 'first');
        
        if isempty(idx)
            continue;
        end

        idx = idx(1);
        for j = idx-1:W
            L = int_row(:,j+1)-int_row(:,i);
            if aspect ~= 0
                [min_st, min_len, cur_sum] = ssp_internal(L, T, ceil((j-i+1)/aspect));
            else
                [min_st, min_len, cur_sum] = ssp_internal(L, T, 0);
            end
            if min_st == 0
                continue;
            end
            curArea = min_len*(j-i+1);
            if curArea < min_area
                min_area = curArea;
                min_rect = [i, min_st, j-i, min_len-1];
                min_sum = cur_sum;
            elseif curArea == min_area && cur_sum>min_sum
                min_rect = [i, min_st, j-i, min_len-1];
                min_sum = cur_sum;
            end
        end 
    end

end


% Smallest subarray with sum greater than a given value
% if L==0, given an array, find the shorest subarray sum up to no less than T
% if L>0 fixed length (L) subarray with sum greater than a given value (T)
function [min_st, min_len, min_sum] = ssp_internal(a, T, len)

    switch len
        case 0
            % if single element larger than T
            if ~isempty(find(a>=T,1))
                [~,idx] = max(a);
                min_st = idx(1);
                min_len = 1;
                min_sum = a(min_st);
            else
                min_st = 1;
                min_len = length(a) + 1;
                min_sum = T - 1;

                st = 1; ed = 1;
                curSum = a(st);

                flag = 1; % 1: ed move 0: st move 
                while st<=length(a) && ed<=length(a)

                    switch flag
                        case 1
                            ed = ed + 1;
                            if ed > length(a)
                                break;
                            else
                                curSum = curSum + a(ed);
                                if curSum >= T
                                    flag = 0;
                                    if ed - st + 1 < min_len
                                        min_len = ed - st + 1;
                                        min_st = st;
                                        min_sum = curSum;
                                    elseif ed - st + 1 == min_len && curSum > min_sum
                                        min_st = st;
                                        min_sum = curSum;
                                    end                            
                                end
                            end
                        case 0
                            st = st + 1;
                            if st > length(a) || st > ed
                                break;
                            else
                                curSum = curSum - a(st-1);
                                if curSum < T
                                    flag = 1;
                                else
                                    if ed - st + 1 < min_len
                                        min_len = ed - st + 1;
                                        min_st = st;
                                        min_sum = curSum;
                                    elseif ed - st + 1 == min_len && curSum > min_sum
                                        min_st = st;
                                        min_sum = curSum;
                                    end                                  
                                end
                            end
                    end
                end        
            end
            if min_len > length(a) % not found at all
               min_st = 0; min_len = length(a) + 1; min_sum = 0;
            end
        otherwise
            min_st = 0; min_sum = 0; min_len = len;
            if len <= length(a)
                acum = [0 cumsum(a')];
                for k = 1:length(a)-len+1
                    cursum = acum(k+len)-acum(k);
                    if cursum >= T && cursum > min_sum
                        min_st = k;
                        min_sum = cursum;
                    end            
                end
            end
    end

end